package org.xbib.interlibrary.sru;

import java.io.IOException;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.http.HttpHelper;
import org.xbib.net.http.client.ExceptionListener;
import org.xbib.net.http.client.HttpResponse;
import org.xbib.net.http.client.ResponseListener;
import org.xbib.net.http.client.TimeoutListener;
import org.xbib.settings.Settings;

public class SRUClientHelper implements AutoCloseable {

    private static final Logger logger = Logger.getLogger(SRUClientHelper.class.getName());

    private final Settings settings;

    private final HttpHelper httpHelper = new HttpHelper();

    public SRUClientHelper(Settings settings) {
        this.settings = settings;
    }

    @Override
    public void close() throws IOException {
        // we do not close the HTTP helper
    }

    public String getIndex() {
        return settings.get("connection.url");
    }

    public void searchCQL(String cql,
                          Consumer<TinyMap.Builder<String, Object>> parameterCompletion,
                          ResponseListener<HttpResponse> responseListener,
                          ExceptionListener exceptionListener,
                          TimeoutListener timeoutListener) throws IOException {
        if (cql == null) {
            logger.log(Level.WARNING, "cql is null");
            return;
        }
        String url = settings.get("connection.url");
        if (url == null) {
            logger.log(Level.WARNING, "no connection URL, check configuration");
            return;
        }
        httpHelper.open(settings);
        httpHelper.get(
                settings.get("connection.url"),
                parameters -> {
                    String version = settings.get("connection.version", "1.2");
                    parameters.put("version", version);
                    parameters.put("operation", "searchRetrieve");
                    parameters.put("recordSchema", settings.get("connection.recordSchema", "dc"));
                    parameters.put("query", cql);
                    parameterCompletion.accept(parameters);
                },
                headers -> {
                },
                responseListener,
                exceptionListener,
                timeoutListener
        );
    }
}
